/* 
 *  GGI/DM default visual allocation
 *
 *  Copyright (c) 1999 Nicolai Haehnle - Locinga@pfaffenwinkel.de
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#define DEBUG		1

#include <stdlib.h>
#include <string.h>

#include <ggi/internal/ggi-dl.h>
#include <ggi/internal/dm_int.h>
#include "default.h"

void free_visual(dmext *hook, dmvisual *dv)
{
	FREE_LIST(dmrect *, &dv->freerects);
	REMOVE_LIST(&hook->dmvis, dv);
	ggiClose(dv->vis);
	free(dv);
}

int expand_visual(dmvisual *dv, int ex, int ey)
{
	int newsx, newsy;
	ggi_mode md;
	dmext *de = dv->parent;
	dmrect *rc;

	GGIDPRINT("default-dm: Expanding visual %p by (%i/%i)\n", dv, ex, ey);

	if (dv->flags & GGIDM_ACQUIRED)
		return -1;
	
	newsx = dv->size.x + ex;
	newsy = dv->size.y + ey;
	
	/* check mode */
	if (dv->flags & GGIDM_VISIBLE) {
		if (ggiCheckGraphMode(dv->vis, de->visible.x, de->visible.y, 
				      newsx, newsy, de->type, &md)) {
			if (de->visible.x && de->visible.x != md.visible.x &&
					     de->visible.y != md.visible.y)
				return -1;
			if (md.virt.x < newsx || md.virt.y < newsy)
				return -1;
			if (md.graphtype != de->type)
				return -1;
		}
	} else {
		if (ggiCheckSimpleMode(dv->vis, newsx, newsy, 1, 
				       de->type, &md)) {
			if (md.virt.x < newsx || md.virt.y < newsy)
				return -1;
			if (md.graphtype != de->type)
				return -1;
		}
	}
	
	ex = md.virt.x - dv->size.x;
	ey = md.virt.y - dv->size.y;
	
	GGIDPRINT("Mode check passed - expanding by (%i/%i)\n", ex, ey);
	
	/* create freerects and update variables */
	rc = (dmrect *)malloc(sizeof(dmrect));
	rc->pos.x = dv->size.x;
	rc->pos.y = 0;
	rc->size.x = ex;
	rc->size.y = dv->size.y;
	merge_rects(&dv->freerects, rc);
	
	rc = (dmrect *)malloc(sizeof(dmrect));
	rc->pos.x = 0;
	rc->pos.y = dv->size.y;
	rc->size.x = dv->size.x+ex;
	rc->size.y = ey;
	merge_rects(&dv->freerects, rc);
	
	dv->size.x += ex;
	dv->size.y += ey;
	
	return 0;
}

dmvisual *create_visual(dmext *hook)
{
	ggi_visual_t vis;
	dmvisual *dv = (dmvisual *)malloc(sizeof(*dv));
	memset(dv, 0, sizeof(*dv));
	
	GGIDPRINT("Creating new memory visual\n");
	
	vis = ggiOpen("display-memory", NULL);
	if (!vis) {
		GGIDPRINT("ggiOpen() failed\n");
		return 0;
	}
	ggiSetFlags(vis, hook->flags);
	vis->opcolor->setpalvec(vis, hook->cm_start, hook->cm_len, 
		hook->colormap);
	
	dv->parent = hook;
	dv->vis = vis;
	
	/* all other variables are set to 0 */
	
	INSERT_LIST(&hook->dmvis, dv);
	
	return dv;
}


/*
 * This physically sets the mode of a visual (using GGI functions)
 * It also sets the GGIDM_ACQUIRED flag
 */
int physically_set_mode(dmext *hook, dmvisual *dv)
{
	ggi_mode md;
	char mode[80];

	GGIDPRINT("physically setting mode\n");

	if (dv->flags & GGIDM_VISIBLE)
		ggiCheckGraphMode(dv->vis, hook->visible.x, hook->visible.y,
				  dv->size.x, dv->size.y, hook->type, &md);
	else
		ggiCheckGraphMode(dv->vis, GGI_AUTO, GGI_AUTO,
				  dv->size.x, dv->size.y, hook->type, &md);
	
	ggiSPrintMode(mode, &md);
	GGIDPRINT("Suggested mode: %s\n", mode);
	
	if (ggiSetMode(dv->vis, &md)) {
		GGIDPRINT("Setmode failed\n");
		return GGIDM_ERR_IO;
	}
	
	return 0;
}


